@@ -8,8 +8,6 @@ use nb;
 #[cfg(feature = "fmt")]
 use core::fmt;
 
-use crate::registers;
-
 #[derive(Debug, Clone, PartialEq)]
 pub enum ChipFifoInfo {
     NoFifo,
@@ -40,37 +38,42 @@ pub enum Parity {
 /// # MMIO version of uart 8250
 ///
 /// **Noticed** This is only tested on the NS16550 compatible UART used in QEMU 5.0 virt machine of RISC-V
-pub struct MmioUart8250<const BASE: usize>;
+pub struct MmioUart8250 {
+    base_address: usize,
+}
 
-impl<const BASE: usize> MmioUart8250<BASE> {
-    const REGS: *const registers::Registers = BASE as *const _;
+impl MmioUart8250 {
+    /// New a uart
+    pub fn new(base_address: usize) -> Self {
+        Self { base_address }
+    }
 
     /// A basic way to init the uart with interrupt enable
     ///
     /// Other way to init can be done by using other methods below
-    pub fn init(clock: usize, baud_rate: usize) {
+    pub fn init(&self, clock: usize, baud_rate: usize) {
         // Enable DLAB and Set divisor
-        Self::toggle_divisor_latch_accessible();
+        self.toggle_divisor_latch_accessible();
         let divisor = clock / (16 * baud_rate);
-        Self::write_dll(divisor as u8);
-        Self::write_dlh((divisor >> 8) as u8);
+        self.write_dll(divisor as u8);
+        self.write_dlh((divisor >> 8) as u8);
 
         // Disable DLAB and set word length 8 bits, no parity, 1 stop bit
-        Self::write_lcr(3);
+        self.write_lcr(3);
         // Enable FIFO
-        Self::write_fcr(1);
+        self.write_fcr(1);
         // No modem control
-        Self::write_mcr(0);
+        self.write_mcr(0);
         // Enable received_data_available_interrupt
-        Self::toggle_received_data_available_interrupt();
+        self.toggle_received_data_available_interrupt();
     }
 
     /// Read a byte from uart
     ///
     /// Return `None` when data is not ready (RBR\[0\] != 1)
-    pub fn read_byte() -> Option<u8> {
-        if Self::is_data_ready() {
-            Some(Self::read_rbr())
+    pub fn read_byte(&self) -> Option<u8> {
+        if self.is_data_ready() {
+            Some(self.read_rbr())
         } else {
             None
         }
@@ -79,8 +82,8 @@ impl<const BASE: usize> MmioUart8250<BASE> {
     /// Write a byte to uart
     ///
     /// Error are not concerned now **MAYBE TODO**
-    pub fn write_byte(byte: u8) {
-        Self::write_thr(byte);
+    pub fn write_byte(&self, byte: u8) {
+        self.write_thr(byte);
     }
 
     /// write THR (offset + 0)
@@ -94,15 +97,15 @@ impl<const BASE: usize> MmioUart8250<BASE> {
     /// > These registers really are the "heart" of serial data communication, and how data is transferred from your software to another computer and how it gets data from other devices. Reading and Writing to these registers is simply a matter of accessing the Port I/O address for the respective UART.
     /// >
     /// > If the receive buffer is occupied or the FIFO is full, the incoming data is discarded and the Receiver Line Status interrupt is written to the IIR register. The Overrun Error bit is also set in the Line Status Register.
-    pub fn write_thr(value: u8) {
-        unsafe { (*Self::REGS).rw[0].write(value) }
+    pub fn write_thr(&self, value: u8) {
+        unsafe { (*cast!(self.base_address)).rw[0].write(value) }
     }
 
     /// read RBR (offset + 0)
     ///
     /// Read Receiver Buffer to get data
-    pub fn read_rbr() -> u8 {
-        unsafe { (*Self::REGS).rw[0].read() }
+    pub fn read_rbr(&self) -> u8 {
+        unsafe { (*cast!(self.base_address)).rw[0].read() }
     }
 
     /// read DLL (offset + 0)
@@ -134,29 +137,29 @@ impl<const BASE: usize> MmioUart8250<BASE> {
     /// | 38400     | 3                    | $00                     | $03                    |
     /// | 57600     | 2                    | $00                     | $02                    |
     /// | 115200    | 1                    | $00                     | $01                    |
-    pub fn read_dll() -> u8 {
-        unsafe { (*Self::REGS).rw[0].read() }
+    pub fn read_dll(&self) -> u8 {
+        unsafe { (*cast!(self.base_address)).rw[0].read() }
     }
 
     /// write DLL (offset + 0)
     ///
     /// set divisor latch low byte in the register
-    pub fn write_dll(value: u8) {
-        unsafe { (*Self::REGS).rw[0].write(value) }
+    pub fn write_dll(&self, value: u8) {
+        unsafe { (*cast!(self.base_address)).rw[0].write(value) }
     }
 
     /// read DLH (offset + 1)
     ///
     /// get divisor latch high byte in the register
-    pub fn read_dlh() -> u8 {
-        unsafe { (*Self::REGS).rw[1].read() }
+    pub fn read_dlh(&self) -> u8 {
+        unsafe { (*cast!(self.base_address)).rw[1].read() }
     }
 
     /// write DLH (offset + 1)
     ///
     /// set divisor latch high byte in the register
-    pub fn write_dlh(value: u8) {
-        unsafe { (*Self::REGS).rw[1].write(value) }
+    pub fn write_dlh(&self, value: u8) {
+        unsafe { (*cast!(self.base_address)).rw[1].write(value) }
     }
 
     /// Read IER (offset + 1)
@@ -179,75 +182,75 @@ impl<const BASE: usize> MmioUart8250<BASE> {
     /// > | 2   | Enable Receiver Line Status Interrupt               |
     /// > | 1   | Enable Transmitter Holding Register Empty Interrupt |
     /// > | 0   | Enable Received Data Available Interrupt            |
-    pub fn read_ier() -> u8 {
-        unsafe { (*Self::REGS).rw[1].read() }
+    pub fn read_ier(&self) -> u8 {
+        unsafe { (*cast!(self.base_address)).rw[1].read() }
     }
 
     /// Write IER (offset + 1)
     ///
     /// Write Interrupt Enable Register to turn on/off interrupts
-    pub fn write_ier(value: u8) {
-        unsafe { (*Self::REGS).rw[1].write(value) }
+    pub fn write_ier(&self, value: u8) {
+        unsafe { (*cast!(self.base_address)).rw[1].write(value) }
     }
 
     /// get whether low power mode (16750) is enabled (IER\[5\])
-    pub fn is_low_power_mode_enabled() -> bool {
-        unsafe { (*Self::REGS).rw[1].read() & 0b0010_0000 != 0 }
+    pub fn is_low_power_mode_enabled(&self) -> bool {
+        unsafe { (*cast!(self.base_address)).rw[1].read() & 0b0010_0000 != 0 }
     }
 
     /// toggle low power mode (16750) (IER\[5\])
-    pub fn toggle_low_power_mode() {
-        unsafe { (*Self::REGS).rw[1].modify(|v| v ^ 0b0010_0000) }
+    pub fn toggle_low_power_mode(&self) {
+        unsafe { (*cast!(self.base_address)).rw[1].modify(|v| v ^ 0b0010_0000) }
     }
 
     /// get whether sleep mode (16750) is enabled (IER\[4\])
-    pub fn is_sleep_mode_enabled() -> bool {
-        unsafe { (*Self::REGS).rw[1].read() & 0b0001_0000 != 0 }
+    pub fn is_sleep_mode_enabled(&self) -> bool {
+        unsafe { (*cast!(self.base_address)).rw[1].read() & 0b0001_0000 != 0 }
     }
 
     /// toggle sleep mode (16750) (IER\[4\])
-    pub fn toggle_sleep_mode() {
-        unsafe { (*Self::REGS).rw[1].modify(|v| v ^ 0b0001_0000) }
+    pub fn toggle_sleep_mode(&self) {
+        unsafe { (*cast!(self.base_address)).rw[1].modify(|v| v ^ 0b0001_0000) }
     }
 
     /// get whether modem status interrupt is enabled (IER\[3\])
-    pub fn is_modem_status_interrupt_enabled() -> bool {
-        unsafe { (*Self::REGS).rw[1].read() & 0b0000_1000 != 0 }
+    pub fn is_modem_status_interrupt_enabled(&self) -> bool {
+        unsafe { (*cast!(self.base_address)).rw[1].read() & 0b0000_1000 != 0 }
     }
 
     /// toggle modem status interrupt (IER\[3\])
-    pub fn toggle_modem_status_interrupt() {
-        unsafe { (*Self::REGS).rw[1].modify(|v| v ^ 0b0000_1000) }
+    pub fn toggle_modem_status_interrupt(&self) {
+        unsafe { (*cast!(self.base_address)).rw[1].modify(|v| v ^ 0b0000_1000) }
     }
 
     /// get whether receiver line status interrupt is enabled (IER\[2\])
-    pub fn is_receiver_line_status_interrupt_enabled() -> bool {
-        unsafe { (*Self::REGS).rw[1].read() & 0b0000_0100 != 0 }
+    pub fn is_receiver_line_status_interrupt_enabled(&self) -> bool {
+        unsafe { (*cast!(self.base_address)).rw[1].read() & 0b0000_0100 != 0 }
     }
 
     /// toggle receiver line status interrupt (IER\[2\])
-    pub fn toggle_receiver_line_status_interrupt() {
-        unsafe { (*Self::REGS).rw[1].modify(|v| v ^ 0b0000_0100) }
+    pub fn toggle_receiver_line_status_interrupt(&self) {
+        unsafe { (*cast!(self.base_address)).rw[1].modify(|v| v ^ 0b0000_0100) }
     }
 
     /// get whether transmitter holding register empty interrupt is enabled (IER\[1\])
-    pub fn is_transmitter_holding_register_empty_interrupt_enabled() -> bool {
-        unsafe { (*Self::REGS).rw[1].read() & 0b0000_0010 != 0 }
+    pub fn is_transmitter_holding_register_empty_interrupt_enabled(&self) -> bool {
+        unsafe { (*cast!(self.base_address)).rw[1].read() & 0b0000_0010 != 0 }
     }
 
     /// toggle transmitter holding register empty interrupt (IER\[1\])
-    pub fn toggle_transmitter_holding_register_empty_interrupt() {
-        unsafe { (*Self::REGS).rw[1].modify(|v| v ^ 0b0000_0010) }
+    pub fn toggle_transmitter_holding_register_empty_interrupt(&self) {
+        unsafe { (*cast!(self.base_address)).rw[1].modify(|v| v ^ 0b0000_0010) }
     }
 
     /// get whether received data available is enabled (IER\[0\])
-    pub fn is_received_data_available_interrupt_enabled() -> bool {
-        unsafe { (*Self::REGS).rw[1].read() & 0b0000_0001 != 0 }
+    pub fn is_received_data_available_interrupt_enabled(&self) -> bool {
+        unsafe { (*cast!(self.base_address)).rw[1].read() & 0b0000_0001 != 0 }
     }
 
     /// toggle received data available (IER\[0\])
-    pub fn toggle_received_data_available_interrupt() {
-        unsafe { (*Self::REGS).rw[1].modify(|v| v ^ 0b0000_0001) }
+    pub fn toggle_received_data_available_interrupt(&self) {
+        unsafe { (*cast!(self.base_address)).rw[1].modify(|v| v ^ 0b0000_0001) }
     }
 
     /// Read IIR (offset + 2)
@@ -282,13 +285,13 @@ impl<const BASE: usize> MmioUart8250<BASE> {
     /// > |            | 1                                 | 1     | 0                                 | Time-out Interrupt Pending (16550 & later)   | Reading Receive Buffer Register(RBR)                                                      |
     /// > |            | 1                                 | 1     | 1                                 | Reserved                                     | N/A                                                                                       |
     /// > | 0          | Interrupt Pending Flag            |       |                                   |                                              |                                                                                           |
-    pub fn read_iir() -> u8 {
-        unsafe { (*Self::REGS).rw[2].read() }
+    pub fn read_iir(&self) -> u8 {
+        unsafe { (*cast!(self.base_address)).rw[2].read() }
     }
 
     /// Read IIR\[7:6\] to get FIFO status
-    pub fn read_fifo_status() -> ChipFifoInfo {
-        match unsafe { (*Self::REGS).rw[2].read() & 0b1100_0000 } {
+    pub fn read_fifo_status(&self) -> ChipFifoInfo {
+        match unsafe { (*cast!(self.base_address)).rw[2].read() & 0b1100_0000 } {
             0 => ChipFifoInfo::NoFifo,
             0b0100_0000 => ChipFifoInfo::Reserved,
             0b1000_0000 => ChipFifoInfo::EnabledNoFunction,
@@ -298,13 +301,13 @@ impl<const BASE: usize> MmioUart8250<BASE> {
     }
 
     /// get whether 64 Byte fifo (16750 only) is enabled (IIR\[5\])
-    pub fn is_64byte_fifo_enabled() -> bool {
-        unsafe { (*Self::REGS).rw[2].read() & 0b0010_0000 != 0 }
+    pub fn is_64byte_fifo_enabled(&self) -> bool {
+        unsafe { (*cast!(self.base_address)).rw[2].read() & 0b0010_0000 != 0 }
     }
 
     /// Read IIR\[3:1\] to get interrupt type
-    pub fn read_interrupt_type() -> InterruptType {
-        match unsafe { (*Self::REGS).rw[2].read() & 0b0000_1110 } {
+    pub fn read_interrupt_type(&self) -> InterruptType {
+        match unsafe { (*cast!(self.base_address)).rw[2].read() & 0b0000_1110 } {
             0 => InterruptType::ModemStatus,
             0b0010 => InterruptType::TransmitterHoldingRegisterEmpty,
             0b0100 => InterruptType::ReceivedDataAvailable,
@@ -316,8 +319,8 @@ impl<const BASE: usize> MmioUart8250<BASE> {
     }
 
     /// get whether interrupt is pending (IIR\[0\])
-    pub fn is_interrupt_pending() -> bool {
-        unsafe { (*Self::REGS).rw[2].read() & 1 != 0 }
+    pub fn is_interrupt_pending(&self) -> bool {
+        unsafe { (*cast!(self.base_address)).rw[2].read() & 1 != 0 }
     }
 
     /// Write FCR (offset + 2) to control FIFO buffers
@@ -341,8 +344,8 @@ impl<const BASE: usize> MmioUart8250<BASE> {
     /// > | 2     | Clear Transmit FIFO         |       |                                   |                         |
     /// > | 1     | Clear Receive FIFO          |       |                                   |                         |
     /// > | 0     | Enable FIFOs                |       |                                   |                         |
-    pub fn write_fcr(value: u8) {
-        unsafe { (*Self::REGS).rw[2].write(value) }
+    pub fn write_fcr(&self, value: u8) {
+        unsafe { (*cast!(self.base_address)).rw[2].write(value) }
     }
 
     /// Read LCR (offset + 3)
@@ -373,30 +376,30 @@ impl<const BASE: usize> MmioUart8250<BASE> {
     /// > |          | 0                        | 1                            | 6 Bits      |               |
     /// > |          | 1                        | 0                            | 7 Bits      |               |
     /// > |          | 1                        | 1                            | 8 Bits      |               |
-    pub fn read_lcr() -> u8 {
-        unsafe { (*Self::REGS).rw[3].read() }
+    pub fn read_lcr(&self) -> u8 {
+        unsafe { (*cast!(self.base_address)).rw[3].read() }
     }
 
     /// Write LCR (offset + 3)
     ///
     /// Write Line Control Register to set DLAB and the serial data protocol
-    pub fn write_lcr(value: u8) {
-        unsafe { (*Self::REGS).rw[3].write(value) }
+    pub fn write_lcr(&self, value: u8) {
+        unsafe { (*cast!(self.base_address)).rw[3].write(value) }
     }
 
     /// get whether DLAB is enabled
-    pub fn is_divisor_latch_accessible() -> bool {
-        unsafe { (*Self::REGS).rw[3].read() & 0b1000_0000 != 0 }
+    pub fn is_divisor_latch_accessible(&self) -> bool {
+        unsafe { (*cast!(self.base_address)).rw[3].read() & 0b1000_0000 != 0 }
     }
 
     /// toggle DLAB
-    pub fn toggle_divisor_latch_accessible() {
-        unsafe { (*Self::REGS).rw[3].modify(|v| v ^ 0b1000_0000) }
+    pub fn toggle_divisor_latch_accessible(&self) {
+        unsafe { (*cast!(self.base_address)).rw[3].modify(|v| v ^ 0b1000_0000) }
     }
 
     /// get parity of used data protocol
-    pub fn get_parity() -> Parity {
-        match unsafe { (*Self::REGS).rw[3].read() & 0b0011_1000 } {
+    pub fn get_parity(&self) -> Parity {
+        match unsafe { (*cast!(self.base_address)).rw[3].read() & 0b0011_1000 } {
             0b0000_0000 => Parity::No,
             0b0000_1000 => Parity::Odd,
             0b0001_1000 => Parity::Even,
@@ -407,47 +410,51 @@ impl<const BASE: usize> MmioUart8250<BASE> {
     }
 
     /// set parity
-    pub fn set_parity(parity: Parity) {
+    pub fn set_parity(&self, parity: Parity) {
         match parity {
-            Parity::No => unsafe { (*Self::REGS).rw[3].modify(|v| (v & 0b1100_0111)) },
+            Parity::No => unsafe {
+                (*cast!(self.base_address)).rw[3].modify(|v| (v & 0b1100_0111))
+            },
             Parity::Odd => unsafe {
-                (*Self::REGS).rw[3].modify(|v| (v & 0b1100_0111) | 0b0000_1000)
+                (*cast!(self.base_address)).rw[3].modify(|v| (v & 0b1100_0111) | 0b0000_1000)
             },
             Parity::Even => unsafe {
-                (*Self::REGS).rw[3].modify(|v| (v & 0b1100_0111) | 0b0001_1000)
+                (*cast!(self.base_address)).rw[3].modify(|v| (v & 0b1100_0111) | 0b0001_1000)
             },
             Parity::Mark => unsafe {
-                (*Self::REGS).rw[3].modify(|v| (v & 0b1100_0111) | 0b0010_1000)
+                (*cast!(self.base_address)).rw[3].modify(|v| (v & 0b1100_0111) | 0b0010_1000)
+            },
+            Parity::Space => unsafe {
+                (*cast!(self.base_address)).rw[3].modify(|v| v | 0b0011_1000)
             },
-            Parity::Space => unsafe { (*Self::REGS).rw[3].modify(|v| v | 0b0011_1000) },
         }
     }
 
     /// get stop bit of used data protocol
     ///
     /// Simply return a u8 to indicate 1 or 1.5/2 bits
-    pub fn get_stop_bit() -> u8 {
-        ((unsafe { (*Self::REGS).rw[3].read() & 0b100 }) >> 2) + 1
+    pub fn get_stop_bit(&self) -> u8 {
+        ((unsafe { (*cast!(self.base_address)).rw[3].read() & 0b100 }) >> 2) + 1
     }
 
     /// set stop bit, only 1 and 2 can be used as `stop_bit`
-    pub fn set_stop_bit(stop_bit: u8) {
+    pub fn set_stop_bit(&self, stop_bit: u8) {
         match stop_bit {
-            1 => unsafe { (*Self::REGS).rw[3].modify(|v| v & 0b1111_1011) },
-            2 => unsafe { (*Self::REGS).rw[3].modify(|v| v | 0b0000_0100) },
+            1 => unsafe { (*cast!(self.base_address)).rw[3].modify(|v| v & 0b1111_1011) },
+            2 => unsafe { (*cast!(self.base_address)).rw[3].modify(|v| v | 0b0000_0100) },
             _ => panic!("Invalid stop bit"),
         }
     }
 
     /// get word length of used data protocol
-    pub fn get_word_length() -> u8 {
-        (unsafe { (*Self::REGS).rw[3].read() & 0b11 }) + 5
+    pub fn get_word_length(&self) -> u8 {
+        (unsafe { (*cast!(self.base_address)).rw[3].read() & 0b11 }) + 5
     }
 
     /// set word length, only 5..=8 can be used as `length`
-    pub fn set_word_length(length: u8) {
+    pub fn set_word_length(&self, length: u8) {
         if (5..=8).contains(&length) {
-            unsafe { (*Self::REGS).rw[3].modify(|v| v | (length - 5)) }
+            unsafe { (*cast!(self.base_address)).rw[3].modify(|v| v | (length - 5)) }
         } else {
             panic!("Invalid word length")
         }
@@ -471,15 +478,15 @@ impl<const BASE: usize> MmioUart8250<BASE> {
     /// > | 2   | Auxiliary Output 1               |
     /// > | 1   | Request To Send                  |
     /// > | 0   | Data Terminal Ready              |
-    pub fn read_mcr() -> u8 {
-        unsafe { (*Self::REGS).rw[4].read() }
+    pub fn read_mcr(&self) -> u8 {
+        unsafe { (*cast!(self.base_address)).rw[4].read() }
     }
 
     /// Write MCR (offset + 4)
     ///
     /// Write Modem Control Register to control flow
-    pub fn write_mcr(value: u8) {
-        unsafe { (*Self::REGS).rw[4].write(value) }
+    pub fn write_mcr(&self, value: u8) {
+        unsafe { (*cast!(self.base_address)).rw[4].write(value) }
     }
 
     /// Read LSR (offset + 5)
@@ -498,40 +505,40 @@ impl<const BASE: usize> MmioUart8250<BASE> {
     /// > | 2   | Parity Error                       |
     /// > | 1   | Overrun Error                      |
     /// > | 0   | Data Ready                         |
-    pub fn read_lsr() -> u8 {
-        unsafe { (*Self::REGS).ro[0].read() }
+    pub fn read_lsr(&self) -> u8 {
+        unsafe { (*cast!(self.base_address)).ro[0].read() }
     }
 
-    pub fn is_received_fifo_error() -> bool {
-        unsafe { (*Self::REGS).ro[0].read() & 0b1000_0000 != 0 }
+    pub fn is_received_fifo_error(&self) -> bool {
+        unsafe { (*cast!(self.base_address)).ro[0].read() & 0b1000_0000 != 0 }
     }
 
-    pub fn is_data_holding_registers_empty() -> bool {
-        unsafe { (*Self::REGS).ro[0].read() & 0b0100_0000 != 0 }
+    pub fn is_data_holding_registers_empty(&self) -> bool {
+        unsafe { (*cast!(self.base_address)).ro[0].read() & 0b0100_0000 != 0 }
     }
 
-    pub fn is_transmitter_holding_register_empty() -> bool {
-        unsafe { (*Self::REGS).ro[0].read() & 0b0010_0000 != 0 }
+    pub fn is_transmitter_holding_register_empty(&self) -> bool {
+        unsafe { (*cast!(self.base_address)).ro[0].read() & 0b0010_0000 != 0 }
     }
 
-    pub fn is_break_interrupt() -> bool {
-        unsafe { (*Self::REGS).ro[0].read() & 0b0001_0000 != 0 }
+    pub fn is_break_interrupt(&self) -> bool {
+        unsafe { (*cast!(self.base_address)).ro[0].read() & 0b0001_0000 != 0 }
     }
 
-    pub fn is_framing_error() -> bool {
-        unsafe { (*Self::REGS).ro[0].read() & 0b0000_1000 != 0 }
+    pub fn is_framing_error(&self) -> bool {
+        unsafe { (*cast!(self.base_address)).ro[0].read() & 0b0000_1000 != 0 }
     }
 
-    pub fn is_parity_error() -> bool {
-        unsafe { (*Self::REGS).ro[0].read() & 0b0000_0100 != 0 }
+    pub fn is_parity_error(&self) -> bool {
+        unsafe { (*cast!(self.base_address)).ro[0].read() & 0b0000_0100 != 0 }
     }
 
-    pub fn is_overrun_error() -> bool {
-        unsafe { (*Self::REGS).ro[0].read() & 0b0000_0010 != 0 }
+    pub fn is_overrun_error(&self) -> bool {
+        unsafe { (*cast!(self.base_address)).ro[0].read() & 0b0000_0010 != 0 }
     }
 
-    pub fn is_data_ready() -> bool {
-        unsafe { (*Self::REGS).ro[0].read() & 0b0000_0001 != 0 }
+    pub fn is_data_ready(&self) -> bool {
+        unsafe { (*cast!(self.base_address)).ro[0].read() & 0b0000_0001 != 0 }
     }
 
     /// Read MSR (offset + 6)
@@ -550,48 +557,48 @@ impl<const BASE: usize> MmioUart8250<BASE> {
     /// > | 2   | Trailing Edge Ring Indicator |
     /// > | 1   | Delta Data Set Ready         |
     /// > | 0   | Delta Clear To Send          |
-    pub fn read_msr() -> u8 {
-        unsafe { (*Self::REGS).ro[1].read() }
+    pub fn read_msr(&self) -> u8 {
+        unsafe { (*cast!(self.base_address)).ro[1].read() }
     }
 
-    pub fn is_carrier_detect() -> bool {
-        unsafe { (*Self::REGS).ro[1].read() & 0b1000_0000 != 0 }
+    pub fn is_carrier_detect(&self) -> bool {
+        unsafe { (*cast!(self.base_address)).ro[1].read() & 0b1000_0000 != 0 }
     }
 
-    pub fn is_ring_indicator() -> bool {
-        unsafe { (*Self::REGS).ro[1].read() & 0b0100_0000 != 0 }
+    pub fn is_ring_indicator(&self) -> bool {
+        unsafe { (*cast!(self.base_address)).ro[1].read() & 0b0100_0000 != 0 }
     }
 
-    pub fn is_data_set_ready() -> bool {
-        unsafe { (*Self::REGS).ro[1].read() & 0b0010_0000 != 0 }
+    pub fn is_data_set_ready(&self) -> bool {
+        unsafe { (*cast!(self.base_address)).ro[1].read() & 0b0010_0000 != 0 }
     }
 
-    pub fn is_clear_to_send() -> bool {
-        unsafe { (*Self::REGS).ro[1].read() & 0b0001_0000 != 0 }
+    pub fn is_clear_to_send(&self) -> bool {
+        unsafe { (*cast!(self.base_address)).ro[1].read() & 0b0001_0000 != 0 }
     }
 
-    pub fn is_delta_data_carrier_detect() -> bool {
-        unsafe { (*Self::REGS).ro[1].read() & 0b0000_1000 != 0 }
+    pub fn is_delta_data_carrier_detect(&self) -> bool {
+        unsafe { (*cast!(self.base_address)).ro[1].read() & 0b0000_1000 != 0 }
     }
 
-    pub fn is_trailing_edge_ring_indicator() -> bool {
-        unsafe { (*Self::REGS).ro[1].read() & 0b0000_0100 != 0 }
+    pub fn is_trailing_edge_ring_indicator(&self) -> bool {
+        unsafe { (*cast!(self.base_address)).ro[1].read() & 0b0000_0100 != 0 }
     }
 
-    pub fn is_delta_data_set_ready() -> bool {
-        unsafe { (*Self::REGS).ro[1].read() & 0b0000_0010 != 0 }
+    pub fn is_delta_data_set_ready(&self) -> bool {
+        unsafe { (*cast!(self.base_address)).ro[1].read() & 0b0000_0010 != 0 }
     }
 
-    pub fn is_delta_clear_to_send() -> bool {
-        unsafe { (*Self::REGS).ro[1].read() & 0b0000_0001 != 0 }
+    pub fn is_delta_clear_to_send(&self) -> bool {
+        unsafe { (*cast!(self.base_address)).ro[1].read() & 0b0000_0001 != 0 }
     }
 
-    pub fn read_sr() -> u8 {
-        unsafe { (*Self::REGS).scratch.read() }
+    pub fn read_sr(&self) -> u8 {
+        unsafe { (*cast!(self.base_address)).scratch.read() }
     }
 
-    pub fn write_sr(value: u8) {
-        unsafe { (*Self::REGS).scratch.write(value) }
+    pub fn write_sr(&self, value: u8) {
+        unsafe { (*cast!(self.base_address)).scratch.write(value) }
     }
 }
 
@@ -599,12 +606,12 @@ impl<const BASE: usize> MmioUart8250<BASE> {
 ///
 /// This is a very simple implementation, based on [rustsbi/rustsbi-qemu](https://github.com/rustsbi/rustsbi-qemu/blob/main/rustsbi-qemu/src/ns16550a.rs#L36-L52)
 #[cfg(feature = "embedded")]
-impl<const BASE: usize> serial::Read<u8> for MmioUart8250<BASE> {
+impl serial::Read<u8> for MmioUart8250 {
     type Error = Infallible;
 
     fn try_read(&mut self) -> nb::Result<u8, Self::Error> {
-        if Self::is_interrupt_pending() {
-            Ok(Self::read_rbr())
+        if self.is_interrupt_pending() {
+            Ok(self.read_rbr())
         } else {
             Err(nb::Error::WouldBlock)
         }
@@ -615,16 +622,16 @@ impl<const BASE: usize> serial::Read<u8> for MmioUart8250<BASE> {
 ///
 /// This is a very simple implementation, based on [rustsbi/rustsbi-qemu](https://github.com/rustsbi/rustsbi-qemu/blob/main/rustsbi-qemu/src/ns16550a.rs#L54-L75)
 #[cfg(feature = "embedded")]
-impl<const BASE: usize> serial::Write<u8> for MmioUart8250<BASE> {
+impl serial::Write<u8> for MmioUart8250 {
     type Error = Infallible;
 
     fn try_write(&mut self, word: u8) -> nb::Result<(), Self::Error> {
-        Self::write_thr(word);
+        self.write_thr(word);
         Ok(())
     }
 
     fn try_flush(&mut self) -> nb::Result<(), Self::Error> {
-        if Self::is_interrupt_pending() {
+        if self.is_interrupt_pending() {
             Ok(())
         } else {
             Err(nb::Error::WouldBlock)
@@ -636,17 +643,17 @@ impl<const BASE: usize> serial::Write<u8> for MmioUart8250<BASE> {
 ///
 /// A simple implementation, may be changed in the future
 #[cfg(feature = "fmt")]
-impl<const BASE: usize> fmt::Write for MmioUart8250<BASE> {
+impl fmt::Write for MmioUart8250 {
     fn write_str(&mut self, s: &str) -> fmt::Result {
         for c in s.as_bytes() {
-            Self::write_thr(*c);
+            self.write_thr(*c);
         }
         Ok(())
     }
 
     fn write_fmt(&mut self, args: fmt::Arguments<'_>) -> fmt::Result {
         for c in args.as_str().unwrap_or("").as_bytes() {
-            Self::write_thr(*c);
+            self.write_thr(*c);
         }
         Ok(())
     }
